'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _immutabilityHelper = require('immutability-helper');

var _immutabilityHelper2 = _interopRequireDefault(_immutabilityHelper);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var getTarget = function getTarget(name, value) {
  return {
    name: name,
    value: (0, _get2.default)(value, name, [])
  };
};

var getAddAction = function getAddAction(position) {
  if (position === 'before') {
    return '$unshift';
  }
  return '$push';
};

var addDelta = function addDelta(name, action) {
  return _defineProperty({}, name, _defineProperty({
    $apply: function $apply(item) {
      return item ? item : [];
    }
  }, action, [{}]));
};

var add = function add(position, name, props, event) {
  var delta = addDelta(name, getAddAction(position));
  event.formData = (0, _immutabilityHelper2.default)(props.formData, delta);
  event.formMeta = (0, _immutabilityHelper2.default)(props.formMeta, delta);
  event.target = getTarget(name, event.formData);
  return event;
};

var changeDelta = function changeDelta(name, idx) {
  return function (value) {
    return _defineProperty({}, name, _defineProperty({}, idx, {
      $set: value
    }));
  };
};

var change = function change(idx, name, props, event) {
  var delta = changeDelta(name, idx);
  event.formData = (0, _immutabilityHelper2.default)(props.formData, delta(event.formData));
  event.formMeta = (0, _immutabilityHelper2.default)(props.formMeta, delta(event.formMeta));
  event.target = getTarget(name, event.formData);
  return event;
};

var removeDelta = function removeDelta(name, idx) {
  return _defineProperty({}, name, {
    $splice: [[idx, 1]]
  });
};

var remove = function remove(idx, name, props, event) {
  var delta = removeDelta(name, idx);
  event.formData = (0, _immutabilityHelper2.default)(props.formData, delta);
  event.formMeta = (0, _immutabilityHelper2.default)(props.formMeta, delta);
  event.target = getTarget(name, event.formData);
  return event;
};

exports.default = {
  add: add,
  change: change,
  remove: remove
};